/*
 * $Id: BulletmlPlayer.java,v 1.6 2002/10/03 15:56:47 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletmlkxml;

import jp.gr.java_conf.abagames.noiz2.*;
import jp.gr.java_conf.abagames.util.*;
import java.io.*;
import java.util.Vector;
import java.util.Random;
import java.awt.Color;

/**
 * Handle a tread.
 *
 * @version $Revision: 1.6 $
 */
public class BulletmlPlayer implements ITime {
  private BarrageManager manager;
  private Screen screen;
  private Ship ship;
  private AttractManager atMng;

  public void init(BarrageManager manager, Screen screen, Ship ship,
                   AttractManager atMng) {
    this.manager = manager;
    manager.setPlayer(this);
    this.screen = screen;
    this.ship = ship;
    this.atMng = atMng;
  }

  /*
   * Draw enemy cubes/bullets/my ship/bonus items/sparks.
   */

  private final int[][] enpb = {
   {1,1,1},{-1,1,1},{-1,-1,1},{1,-1,1},
   {1,1,-1},{-1,1,-1},{-1,-1,-1},{1,-1,-1},
  };
  private final int[][] enpc = {
    {0,1,2,3},{4,5,6,7},{0,1,5,4},{1,2,6,5},{2,3,7,6},{3,0,4,7}
  };

  private final int[][] enp = new int[8][2];

  //private final int[] ENEMY_COLOR = {
  //  0xffffaaaa, 0xffffeeaa, 0xffffaaee, 0xffeeffee,
  //};
  private final Color[] ENEMY_COLOR = {
    new Color(0xffaaaa), new Color(0xffeeaa), new Color(0xffaaee), new Color(0xeeffee),
  };

  public final void drawEnemy(int x, int y, int cnt, int shield, int type) {
    int size, d1, d2;
    int ty;
    int eci1, eci2;
    if ( cnt < 32 ) size = cnt<<1;
    else size = 64;
    size += (shield<<5);
    d1 = (cnt*5)&(SCTable.TABLE_SIZE-1);
    d2 = (cnt*7)&(SCTable.TABLE_SIZE-1);
    for ( int i=0 ; i<8 ; i++ ) {
      enp[i][0] = (enpb[i][0]*SCTable.costbl[d1] -
                   enpb[i][1]*SCTable.sintbl[d1]) * size;
      ty        = (enpb[i][0]*SCTable.sintbl[d1] +
                   enpb[i][1]*SCTable.costbl[d1]);
      enp[i][1] = (((ty        *SCTable.costbl[d2])>>8) -
                     enpb[i][2]*SCTable.sintbl[d2]) * size;
      enp[i][0] >>= 4;
      enp[i][1] >>= 4;
    }
    screen.setColor(ENEMY_COLOR[type]);
    for ( int i=0 ; i<6 ; i++ ) {
      for ( int j=0 ; j<4 ; j++ ) {
        eci1 = enpc[i][j];
        eci2 = enpc[i][(j+1)&3];
        //screen.drawLine((x+enp[eci1][0])>>8, (y+enp[eci1][1])>>8,
        //                (x+enp[eci2][0])>>8, (y+enp[eci2][1])>>8, ENEMY_COLOR[type]);
        screen.drawLine((x+enp[eci1][0])>>8, (y+enp[eci1][1])>>8,
                        (x+enp[eci2][0])>>8, (y+enp[eci2][1])>>8);
      }
    }
  }

  //private final int bulletColor[] = {
  //  0xff006666, 0xff660066, 0xff666600
  //};
  private final Color bulletColor[] = {
    new Color(0xaaffff), new Color(0xffaaff), new Color(0xffffaa)
  };

  private final Color[] wakeColor = {
    new Color(0x448844), new Color(0x336633), new Color(0x224422), new Color(0x112211),
  };
  private final int wakeCnt = 64;

  public final void drawBullet(int x, int y, int px, int py, int sx, int sy, int cl, int cnt) {
    if ( cnt < wakeCnt ) {
      //int wc = (wakeCnt-cnt)<<2;
      //wc = ((wc>>1)<<16 | (wc>>0)<<8 | (wc>>1) | 0xff000000);
      //screen.drawLine(sx>>8, sy>>8, px>>8, py>>8, wc);
      int wc = cnt>>4;
      screen.setColor(wakeColor[wc]);
      screen.drawLine(sx>>8, sy>>8, px>>8, py>>8);
    }
    //screen.drawThickLine(x>>8, y>>8, px>>8, py>>8, bulletColor[cl%3], 0xffffffff);
    screen.setColor(bulletColor[cl%3]);
    //screen.drawThickLine(x>>8, y>>8, px>>8, py>>8);
    screen.drawLine(x>>8, y>>8, px>>8, py>>8);
  }

  /*private final int SHIP_COLOR1 = 0xffffffff;
  private final int SHIP_COLOR2 = 0xff22aaee;
  private final int LOCK_COLOR1 = 0xff886622;
  private final int LOCK_COLOR2 = 0xff442211;*/
  private final Color SHIP_COLOR1 = new Color(0xffffff);
  private final Color SHIP_COLOR2 = new Color(0x22aaee);
  private final Color LOCK_COLOR1 = new Color(0x886622);
  private final Color LOCK_COLOR2 = new Color(0x442211);

  private int shipX, shipY;

  public final void drawShip(int x, int y, int px, int py, int cnt) {
    shipX = x>>8; shipY = y>>8;
    //screen.drawThickLine(shipX, shipY, px>>8, py>>8, SHIP_COLOR1, SHIP_COLOR2);
    screen.setColor(SHIP_COLOR2);
    screen.drawThickLine(shipX, shipY, px>>8, py>>8);
    int ly = (cnt&31)*12;
    //screen.drawLine(shipX, shipY, shipX, shipY-ly, LOCK_COLOR1);
    //screen.drawLine(shipX, shipY-ly, shipX, 0, LOCK_COLOR2);
    screen.setColor(LOCK_COLOR1);
    screen.drawLine(shipX, shipY, shipX, shipY-ly);
    screen.setColor(LOCK_COLOR2);
    screen.drawLine(shipX, shipY-ly, shipX, 0);
  }

  //private final int HL_COLOR1 = 0xff4477aa;
  //private final int HL_COLOR2 = 0xff44aaff;
  //private final int LOCK_WAKE_COLOR = 0xff224488;
  private final Color HL_COLOR1 = new Color(0x4477aa);
  private final Color HL_COLOR2 = new Color(0x44aaff);
  private final Color LOCK_WAKE_COLOR = new Color(0x224488);

  public final void drawHormingLaser(int x, int y, int px, int py, int cnt) {
    //screen.drawLine(shipX, shipY, px>>8, py>>8, LOCK_WAKE_COLOR);
    screen.setColor(LOCK_WAKE_COLOR);
    screen.drawLine(shipX, shipY, px>>8, py>>8);
    //screen.drawThickLine(x>>8, y>>8, px>>8, py>>8, HL_COLOR1, HL_COLOR2);
    screen.setColor(HL_COLOR2);
    screen.drawThickLine(x>>8, y>>8, px>>8, py>>8);
  }

  //private final int BONUS_COLOR = 0xff44ee22;
  private final Color BONUS_COLOR = new Color(0x44ee22);

  public final void drawBonus(int x, int y, int cnt) {
    int d, ox, oy, sx, sy;
    if ( cnt < 16 ) d = (32-cnt)*cnt;
    else d = 16*16+cnt*16;
    d &= (SCTable.TABLE_SIZE-1);
    ox = SCTable.costbl[d]>>6; oy = SCTable.sintbl[d]>>6;
    sx = x>>8; sy = y>>8;
    //screen.drawLine(sx-ox, sy-oy, sx+ox, sy+oy, BONUS_COLOR);
    //screen.drawLine(sx+oy, sy-ox, sx-oy, sy+ox, BONUS_COLOR);
    screen.setColor(BONUS_COLOR);
    screen.drawLine(sx-ox, sy-oy, sx+ox, sy+oy);
    screen.drawLine(sx+oy, sy-ox, sx-oy, sy+ox);
  }

  //private final int SPARK_COLOR = 0xffddff33;
  private final Color SPARK_COLOR = new Color(0xddff33);

  public final void drawSpark(int x, int y) {
    //screen.drawDot(x>>8, y>>8, SPARK_COLOR);
    screen.setColor(SPARK_COLOR);
    screen.drawDot(x>>8, y>>8);
  }


  public void hitShip() {
    ship.hit();
  }

  public void lockShip(BulletImpl bl) {
    ship.lock(bl);
  }

  public void clearStage() {
    atMng.initClear();
  }

  public void gameover() {
    atMng.initGameover();
  }

  /*
   * Methods to handle a thread.
   */
  private Timer timer;
  private int interval = BarrageManager.INTERVAL_BASE;

  public void start() {
    timer = new Timer(this);
    activate();
  }

  public void setInterval(int itv) {
    interval = itv;
    timer.setDuration(itv);
  }

  public void activate() {
    //timer.activate(interval, false);
    timer.activate(interval, true);
  }

  public void deactive() {
    if ( timer != null ) {
      timer.deactivate();
    }
  }

  public void terminate() {
    timer.terminate();
  }

  private synchronized void waitFrame() {
    try {
      wait();
    } catch ( InterruptedException e ) {}
  }

  public synchronized final void notifyFrame() {
    notifyAll();
  }

  public final void wakeUp() {
    loop();
    waitFrame();
  }

  private long prvTickCount = 0;

  /**
   * Thread main loop.
   */
  private void loop() {
    long nowTick = System.currentTimeMillis();
    int frame = (int)(nowTick-prvTickCount) / interval;
    if ( frame <= 0 ) {
      frame = 1;
      prvTickCount = nowTick;
    } else if ( frame > 5 ) {
      frame = 5;
      prvTickCount = nowTick;
    } else {
      prvTickCount += frame*interval;
    }
    // Movements.
    for ( int i=0 ; i<frame ; i++ ) {
      switch ( atMng.state ) {
      case AttractManager.TITLE:
        atMng.moveTitle();
        if ( atMng.isInDemo() ) {
          manager.addBullets();
          manager.moveBullets();
        }
        break;
      case AttractManager.IN_GAME:
        ship.move();
        manager.addBullets();
        manager.moveBullets();
        break;
      case AttractManager.STAGE_CLEAR:
        atMng.moveClear();
        ship.move();
        manager.moveBullets();
        break;
      case AttractManager.GAME_OVER:
        atMng.moveGameover();
        ship.moveSpark();
        manager.moveBullets();
        break;
      }
    }
    // Display.
    screen.clear();
    switch ( atMng.state ) {
    case AttractManager.TITLE:
      if ( atMng.isInDemo() ) {
        manager.drawBullets();
      }
      atMng.drawTitle();
      break;
    case AttractManager.IN_GAME:
      ship.draw();
      manager.drawScene();
      manager.drawBullets();
      ship.drawScore();
      break;
    case AttractManager.STAGE_CLEAR:
      ship.draw();
      manager.drawBullets();
      ship.drawScore();
      manager.drawScene();
      atMng.drawClear();
      break;
    case AttractManager.GAME_OVER:
      ship.drawSpark();
      manager.drawBullets();
      ship.drawScore();
      atMng.drawGameover();
      break;
    }
    screen.swap();
  }
}
