/*
 * $Id: AttractManager.java,v 1.5 2002/10/03 15:56:48 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import jp.gr.java_conf.abagames.bulletmlkxml.*;
import java.awt.*;
import java.applet.*;
import java.util.Random;

/**
 * Handle title/gameover.
 *
 * @version $Revision: 1.5 $
 */
public class AttractManager {
  public static final int TITLE = 0;
  public static final int IN_GAME = 1;
  public static final int GAME_OVER = 2;
  public static final int STAGE_CLEAR = 3;

  public int state;
  private int cnt;

  public static final int STAGE_NUM = 11;
  private int[] stageX = new int [STAGE_NUM];
  private int[] stageY = new int [STAGE_NUM];
  public static final int SCENE_NUM = 10;

  private BarrageManager manager;
  private Screen screen;
  private Ship ship;
  private PrefManager prMng;

  private final int BOX_SIZE = 32;

  private final int TITLE_IMAGE_NUM = 5;
  public Image[] titleImage = new Image[TITLE_IMAGE_NUM];

  public AttractManager(
   BarrageManager manager, Screen screen, Ship ship, PrefManager prMng) {
    this.manager = manager;
    this.screen = screen;
    this.ship = ship;
    this.prMng = prMng;
    int s = 0;
    int y = Screen.HEIGHT/3;
    for ( int i=0 ; i<4 ; i++, y += BOX_SIZE ) {
      int x = Screen.WIDTH/7*4 - BOX_SIZE*i/2;
      for ( int j=0 ; j<=i ; j++, s++, x += BOX_SIZE ) {
        stageX[s] = x; stageY[s] = y;
      }
    }
    stageX[10] = stageX[0]; stageY[10] = y;
  }

  /*
   * Load title images.
   */

  //private final String IMAGES_DIR = "images/";
  private final String IMAGES_DIR = "/jp/gr/java_conf/abagames/noiz2/images/";
  //private final String IMAGES_DIR = "e:\\usr\\noiz2_cosmo\\jar_image\\jp\\gr\\java_conf\\abagames\\noiz2\\images\\";

  public void loadImages() {
    Toolkit tk = Toolkit.getDefaultToolkit();
    titleImage[0] = tk.createImage(IMAGES_DIR+"n.gif");
    titleImage[1] = tk.createImage(IMAGES_DIR+"o.gif");
    titleImage[2] = tk.createImage(IMAGES_DIR+"i.gif");
    titleImage[3] = tk.createImage(IMAGES_DIR+"z.gif");
    titleImage[4] = tk.createImage(IMAGES_DIR+"2.gif");
  }

  /*public void loadImages(Applet applet) {
    Class appletClass = applet.getClass();
    titleImage[0] = applet.getImage(appletClass.getResource("images/n.gif"));
    titleImage[1] = applet.getImage(appletClass.getResource("images/o.gif"));
    titleImage[2] = applet.getImage(appletClass.getResource("images/i.gif"));
    titleImage[3] = applet.getImage(appletClass.getResource("images/z.gif"));
    titleImage[4] = applet.getImage(appletClass.getResource("images/2.gif"));
  }*/

  /*
   * Initialize title/game/gameover/stageclear.
   */
  private int selectedStage;
  private boolean inDemo;
  private int stBtnY;

  public void initTitle() {
    state = TITLE;
    selectedStage = -1;
    inDemo = false;
    stBtnY = Screen.HEIGHT;
    //screen.setBlur(true);
    msPrsd = false;
    Bonus.setInhaleMag(1.0f);
    cnt = 0;
    screen.setAttractManager(this);

    manager.initStageAsDemo(0);
    inDemo = true;
  }

  public void initGame(int stage) {
    state = IN_GAME;
    manager.initStage(stage);
    //screen.setBlur(false);
    screen.setAttractManager(null);
  }

  public void initGameover() {
    state = GAME_OVER;
    msPrsd = false;
    cnt = 0;
  }

  public void initClear() {
    state = STAGE_CLEAR;
    msPrsd = false;
    Bonus.setInhaleMag(8.0f);
    cnt = 0;
  }


  public boolean isInDemo() {
    return inDemo;
  }

  private int msX, msY;
  private boolean msPrsd = false;

  public void setPos(int x, int y) {
    msX = x; msY = y;
  }

  public void mousePressed() {
    msPrsd = true;
  }

  private Random rnd = new Random();

  public void moveTitle() {
    cnt++;
    if ( msPrsd ) {
      msPrsd = false;
      if ( selectedStage >= 0 && msY > stBtnY ) {
        initGame(selectedStage);
        return;
      }
      for ( int i=0 ; i<STAGE_NUM ; i++ ) {
        if ( msX > stageX[i] && msX < stageX[i]+BOX_SIZE &&
             msY > stageY[i] && msY < stageY[i]+BOX_SIZE && prMng.isOpened(i) ) {
          selectedStage = i;
          if ( i == 10 ) {
            stageStr = "ENDLESS";
          } else {
            stageStr = "STAGE " + (i+1);
          }
          hiscoreStr = Integer.toString(prMng.getStageScore(i));
          manager.initStageAsDemo(i);
          inDemo = true;
        }
      }
    }
    if ( selectedStage >= 0 ) {
      if ( stBtnY > Screen.HEIGHT-32 ) {
        stBtnY--;
      }
    }
  }

  private void checkHiscore() {
    int sc = ship.getScore();
    if ( sc > prMng.getStageScore(selectedStage) ) {
      prMng.setStageScore(selectedStage, sc);
    }
  }

  public void moveGameover() {
    cnt++;
    if ( cnt > 900 || ( cnt > 128 && msPrsd ) ) {
      checkHiscore();
      prMng.save();
      initTitle();
    }
    msPrsd = false;
  }

  public void moveClear() {
    cnt++;
    if ( cnt > 900 || ( cnt > 128 && msPrsd ) ) {
      checkHiscore();
      prMng.clearStage(selectedStage);
      prMng.save();
      initTitle();
    }
    msPrsd = false;
  }

  /*private final int BOX_COLOR1 = 0xffffddbb;
  private final int BOX_COLOR2 = 0xffeeeeee;
  private final int LETTER_COLOR = 0xffffaa55;*/
  private final Color BOX_COLOR1 = new Color(0xffddbb);
  private final Color BOX_COLOR2 = new Color(0xeeeeee);
  private final Color LETTER_COLOR = new Color(0xffaa55);


  private String stageStr, hiscoreStr;

  public void drawTitle() {
    for ( int i=0 ; i<STAGE_NUM ; i++ ) {
      if ( !prMng.isOpened(i) ) continue;
      int x = stageX[i], y = stageY[i];
      if ( i != selectedStage ) {
        /*screen.drawLine(x+2         , y         , x+BOX_SIZE-2, y         , BOX_COLOR1);
        screen.drawLine(x+BOX_SIZE, y+2         , x+BOX_SIZE, y+BOX_SIZE-2, BOX_COLOR1);
        screen.drawLine(x+BOX_SIZE-2, y+BOX_SIZE, x+2         , y+BOX_SIZE, BOX_COLOR1);
        screen.drawLine(x         , y+BOX_SIZE-2, x         , y+2         , BOX_COLOR1);*/
        screen.setColor(BOX_COLOR1);
        screen.drawLine(x+2         , y         , x+BOX_SIZE-2, y         );
        screen.drawLine(x+BOX_SIZE, y+2         , x+BOX_SIZE, y+BOX_SIZE-2);
        screen.drawLine(x+BOX_SIZE-2, y+BOX_SIZE, x+2         , y+BOX_SIZE);
        screen.drawLine(x         , y+BOX_SIZE-2, x         , y+2         );
      } else {
        /*screen.drawThickLine(x         , y         , x+BOX_SIZE, y         , BOX_COLOR1, BOX_COLOR2);
        screen.drawThickLine(x+BOX_SIZE, y         , x+BOX_SIZE, y+BOX_SIZE, BOX_COLOR1, BOX_COLOR2);
        screen.drawThickLine(x+BOX_SIZE, y+BOX_SIZE, x         , y+BOX_SIZE, BOX_COLOR1, BOX_COLOR2);
        screen.drawThickLine(x         , y+BOX_SIZE, x         , y         , BOX_COLOR1, BOX_COLOR2);*/
        screen.setColor(BOX_COLOR2);
        screen.drawThickLine(x         , y         , x+BOX_SIZE, y         );
        screen.drawThickLine(x+BOX_SIZE, y         , x+BOX_SIZE, y+BOX_SIZE);
        screen.drawThickLine(x+BOX_SIZE, y+BOX_SIZE, x         , y+BOX_SIZE);
        screen.drawThickLine(x         , y+BOX_SIZE, x         , y         );
      }
      if ( i < 9 ) {
        LetterRender.drawString(Integer.toString(i+1), x+BOX_SIZE/4, y+BOX_SIZE/4, 8, LETTER_COLOR);
      } else if ( i == 9 ) {
        LetterRender.drawString(Integer.toString(i+1), x, y+BOX_SIZE/4, 7, LETTER_COLOR);
      } else {
        LetterRender.drawString("0", x+BOX_SIZE/4, y+BOX_SIZE/4, 8, LETTER_COLOR);
      }
    }
    if ( selectedStage >= 0 ) {
      if ( (cnt&63) < 32 ) {
        LetterRender.drawString("START", Screen.WIDTH/3, stBtnY+6, 8, LETTER_COLOR);
      }
      //screen.drawLine(0, stBtnY, Screen.WIDTH, stBtnY, BOX_COLOR1);
      screen.setColor(BOX_COLOR1);
      screen.drawLine(0, stBtnY, Screen.WIDTH, stBtnY);
      LetterRender.drawStringFromRight(stageStr, Screen.WIDTH, 40, 10, LETTER_COLOR);
      LetterRender.drawStringFromRight(hiscoreStr, Screen.WIDTH, 72, 7, LETTER_COLOR);
    } else {
      LetterRender.drawString("VER " + Noiz2.VERSION, 4, 40, 4, LETTER_COLOR);
      LetterRender.drawString("SELECT STAGE", 4, Screen.HEIGHT-28, 8, LETTER_COLOR);
    }
  }

  public void drawTitleBoard(Graphics g) {
    int x = 4, y = 4;
    for ( int i=0 ; i<TITLE_IMAGE_NUM ; i++, x+=33 ) {
      g.drawImage(titleImage[i], x, y, null);
    }
  }

  public void drawGameover() {
    int y;
    if ( cnt < 128 ) {
      y = Screen.HEIGHT/3*cnt/128;
    } else {
      y = Screen.HEIGHT/3;
    }
    LetterRender.drawString("GAMEOVER", Screen.WIDTH/4, y, 8, LETTER_COLOR);
  }

  public void drawClear() {
    int y;
    if ( cnt < 128 ) {
      y = Screen.HEIGHT - Screen.HEIGHT/3*2*cnt/128;
    } else {
      y = Screen.HEIGHT/3;
    }
    LetterRender.drawString("STAGE CLEAR", Screen.WIDTH/8, y, 8, LETTER_COLOR);
  }
}
