/*
 * $Id: Noiz2.java,v 1.7 2002/10/03 15:56:48 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import jp.gr.java_conf.abagames.bulletmlkxml.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
//import java.applet.*;
import jp.kyocera.monalisa.awt.*;
import jp.kyocera.monalisa.sys.mgr.*;

/**
 * Noiz2 bootstrap.
 *
 * @version $Revision: 1.7 $
 */
public class Noiz2 extends MApp {
  private Noiz2Frame mframe;

  public static final String VERSION = "0.21B";
  public static final int VERSION_NUM = 20;

  private Screen screen;
  private BarrageManager manager;
  private BulletmlPlayer player;
  private Ship ship;
  private AttractManager atMng;
  private PrefManager prMng;

  public Noiz2(String name, String[] args) {
    super(name, args);
    firstRepaint = false;
  }

  // Is this application executed as the application or the applet?
  private boolean standAlone = false;

  public void setStandalone(boolean f) {
    standAlone = f;
  }

  protected MFrame create(MApp main) {
    mframe = new Noiz2Frame("/jp/gr/java_conf/abagames/noiz2/ico16_noiz2.gif", "Noiz2");
    mframe.init(this);
    mframe.setApp(this);
    return mframe;
  }

  private void initFirst() throws Exception {
    prMng = new PrefManager();
    if ( standAlone ) {
      manager = new BarrageManager();
      prMng.setUseFile(true);
    } else {
      //manager = new BarrageManager(this.getClass());
      //prMng.setUseFile(false);
    }

    prMng.init();
    prMng.load();

    screen = new Screen();
    player = new BulletmlPlayer();
    ship = new Ship(manager, player, prMng);
    manager.setShip(ship);
    atMng = new AttractManager(manager, screen, ship, prMng);
    if ( standAlone ) {
      atMng.loadImages();
    } else {
      //atMng.loadImages(this);
    }
    player.init(manager, screen, ship, atMng);
    LetterRender.setScreen(screen);
  }

  private void activeGame() {
    if ( player != null ) {
      player.activate();
    }
  }

  private void deactiveGame() {
    if ( player != null ) {
      player.deactive();
    }
  }

  protected void start(int state) {
    //if ( state == MApp.SUSPENDED ) {
    //  activeGame();
    //} else {
      standAlone = true;
      try {
        initFirst();
      } catch ( Exception e ) {
        e.printStackTrace();
        manager = null;
        closeApp(true);
      }
      ship.init();
      atMng.initTitle();

      firstRepaint = true;
    //}
    mframe.repaint();
  }

  protected void pause(int state, boolean unconditional) {
    release();
  }

  protected void suspend(int state) {
    //deactiveGame();
    release();
  }

  protected void release() {
    deactiveGame();
    if ( mframe != null ) {
      mframe.close();
      mframe = null;
    }
    //AppManager.setStatus(getApplicationID(), AppManager.APPMGR_STAT_NORMAL);
  }

  private boolean firstRepaint;
  private Panel panel;

  public void paint() {
    for ( int i=0 ; i<5 ; i++ ) {
      panel = mframe.getPanel();
      if ( panel != null ) break;
      try {
        Thread.sleep(500);
      } catch ( InterruptedException e ) {}
    }

    if ( screen != null && panel != null ) {
      screen.makeBuffer(panel);
      if ( manager != null && firstRepaint ) {
        firstRepaint = false;
        player.start();

        //AppManager.setStatus(getApplicationID(), AppManager.APPMGR_STAT_PRIVILEGE);
      }
    }
  }

  public final boolean handleEvent(Event event) {
    if ( manager != null && event.target == panel ) {
      switch ( event.id ) {
      case Event.MOUSE_DRAG:
        atMng.mousePressed();
        switch ( atMng.state ) {
        case AttractManager.TITLE:
          atMng.setPos(event.x, (event.y-16));
          return true;
        case AttractManager.IN_GAME:
        case AttractManager.STAGE_CLEAR:
          ship.setPos(event.x<<8, (event.y-16)<<8);
          return true;
        }
      }
    }
    return false;
  }
}
