/*
 * $Id: Ship.java,v 1.6 2002/10/03 15:56:49 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import jp.gr.java_conf.abagames.bulletmlkxml.*;
import java.util.Random;
import java.awt.Color;

/**
 * Handle my ship.
 *
 * @version $Revision: 1.6 $
 */
public class Ship {
  public int x, y, px, py;
  private int cnt;

  /*
   * Horming lasers data.
   */
  private final int HORMING_LASER_NUM = 8;
  private HormingLaser[] hl = new HormingLaser[HORMING_LASER_NUM];
  private int hlIdx = HORMING_LASER_NUM, hlCnt = 0;

  /*
   * Bonus items data.
   */
  private final int BONUS_NUM = 256;
  private Bonus[] bonus = new Bonus[BONUS_NUM];
  private int bnIdx = BONUS_NUM;

  /*
   * Sparks data.
   */
  private final int SPARK_NUM = 64;
  private Spark[] spark = new Spark[SPARK_NUM];
  private int spIdx = SPARK_NUM;

  private BarrageManager manager;
  private BulletmlPlayer player;
  private PrefManager prMng;

  private int score, bonusScore, extendScore, sceneScore;
  private final int BONUS_SCORE_INIT = 10;
  private final int EXTEND_EVERY = 100000;
  private int left, invCnt, extCnt;
  private final int INVINCIBLE_TIME = 180;

  public Ship(BarrageManager manager, BulletmlPlayer player, PrefManager prMng) {
    this.manager = manager;
    this.player = player;
    this.prMng = prMng;
    for ( int i=0 ; i<HORMING_LASER_NUM ; i++ ) {
      hl[i] = new HormingLaser(player, this);
    }
    for ( int i=0 ; i<BONUS_NUM ; i++ ) {
      bonus[i] = new Bonus(player, this);
    }
    for ( int i=0 ; i<SPARK_NUM ; i++ ) {
      spark[i] = new Spark(player);
    }
  }

  public void init() {
    for ( int i=0 ; i<HORMING_LASER_NUM ; i++ ) {
      hl[i].x = HormingLaser.NOT_EXIST;
    }
    for ( int i=0 ; i<BONUS_NUM ; i++ ) {
      bonus[i].x = Bonus.NOT_EXIST;
    }
    for ( int i=0 ; i<SPARK_NUM ; i++ ) {
      spark[i].x = Spark.NOT_EXIST;
    }
    mouseX = px = x = (Screen.WIDTH/2)<<8;
    mouseY = py = y = (Screen.HEIGHT/4*3)<<8;
    cnt = 0;
    hlCnt = 0;
    score = 0; bonusScore = BONUS_SCORE_INIT;
    left = 2; invCnt = INVINCIBLE_TIME;
    extendScore = EXTEND_EVERY; extCnt = 0;
    sceneScore = 0;
  }

  private int mouseX, mouseY;

  public void setPos(int x, int y) {
    mouseX = x; mouseY = y;
  }

  public final void moveSpark() {
    for ( int i=0 ; i<SPARK_NUM ; i++ ) {
      if ( spark[i].x != Spark.NOT_EXIST ) {
        spark[i].move();
      }
    }
  }

  public final void move() {
    px = x; py = y;
    x = mouseX; y = mouseY;
    if ( px == x && py == y ) {
      py = y+1;
    }

    for ( int i=0 ; i<HORMING_LASER_NUM ; i++ ) {
      if ( hl[i].x != HormingLaser.NOT_EXIST ) {
        hl[i].move();
      }
    }
    for ( int i=0 ; i<BONUS_NUM ; i++ ) {
      if ( bonus[i].x != Bonus.NOT_EXIST ) {
        bonus[i].move();
      }
    }
    moveSpark();

    if ( score >= extendScore ) {
      if ( left < 9 ) {
        left++;
        extCnt = 96;
      }
      extendScore += EXTEND_EVERY;
    }

    if ( hlCnt > 0 ) hlCnt--;
    if ( invCnt > 0 ) invCnt--;
    if ( extCnt > 0 ) extCnt--;
    cnt++;
  }

  public final void drawSpark() {
    for ( int i=0 ; i<SPARK_NUM ; i++ ) {
      if ( spark[i].x != Spark.NOT_EXIST ) {
        spark[i].draw();
      }
    }
  }

  public final void draw() {
    drawSpark();
    for ( int i=0 ; i<HORMING_LASER_NUM ; i++ ) {
      if ( hl[i].x != HormingLaser.NOT_EXIST ) {
        hl[i].draw();
      }
    }
    for ( int i=0 ; i<BONUS_NUM ; i++ ) {
      if ( bonus[i].x != Bonus.NOT_EXIST ) {
        bonus[i].draw();
      }
    }
    player.drawShip(x, y, px, py, cnt);
  }

  //private final int SCORE_COLOR = 0xffaaffee;
  //private final int BONUS_SCORE_COLOR = 0xffeeffaa;
  //private final int LEFT_COLOR = 0xffeeaaff;
  private final Color SCORE_COLOR = new Color(0xaaffee);
  private final Color BONUS_SCORE_COLOR = new Color(0xeeffaa);
  private final Color LEFT_COLOR = new Color(0xeeaaff);

  /**
   * Draw the score and the number of my ships.
   */
  public final void drawScore() {
    LetterRender.drawStringFromRight(Integer.toString(score),
     Screen.WIDTH/3*2, 4, 7, SCORE_COLOR);
    LetterRender.drawStringFromRight(Integer.toString(bonusScore),
     Screen.WIDTH, 4, 5, BONUS_SCORE_COLOR);
    if ( invCnt > 0 || extCnt > 0 ) {
      LetterRender.drawStringFromRight("LEFT-" + Integer.toString(left),
       Screen.WIDTH, 24, 8, LEFT_COLOR);
    }
  }

  /**
   * My ship was destroyed.
   */
  public void hit() {
    if ( invCnt > 0 || left < 0 ) return;
    left--; invCnt = INVINCIBLE_TIME;
    bonusScore = BONUS_SCORE_INIT;
    if ( left < 0 ) {
      player.gameover();
      invCnt = extCnt = 0;
    } else {
      manager.clearZakoBullets();
    }
    for ( int i=0 ; i<64 ; i++ ) {
      addSpark(x, y, rnd.nextInt()%2048, rnd.nextInt()%2048);
    }
  }

  /**
   * A horming laser locks the enemy.
   */
  public void lock(BulletImpl bl) {
    if ( hlCnt > 0 ) return;
    hlIdx--; if ( hlIdx < 0 ) hlIdx = HORMING_LASER_NUM-1;
    hl[hlIdx].set(bl, x, y);
    bl.locked--;
    hlCnt = 12;
  }

  private Random rnd = new Random();

  /**
   * Bullets change to bonus items.
   */
  public void addBonusWiped(int x, int y, int mx, int my) {
    bnIdx--; if ( bnIdx < 0 ) bnIdx = BONUS_NUM-1;
    bonus[bnIdx].set(x, y, mx, my);
  }

  public final void addSpark(int x, int y, int mx, int my) {
    spIdx--; if ( spIdx < 0 ) spIdx = SPARK_NUM-1;
    spark[spIdx].set(x, y, mx, my);
  }

  public int clearScene(int stage, int scene) {
    int ssof = prMng.clearScene(sceneScore, stage, scene);
    sceneScore = 0;
    return ssof;
  }

  private void addScore(int sc) {
    score += sc;
    sceneScore += sc;
  }

  private final int BULLET_WIPE_WIDTH = 5120;
  private final int enemyScore[] = {100, 300, 1000, 10000};

  public void destroyEnemy(int x, int y, int type) {
    addScore(enemyScore[type]);
    manager.wipeBullets(x, y, (type+1)*BULLET_WIPE_WIDTH);
  }

  /**
   * A horming laser hits a enemy.
   */
  public void hitLaser(int x, int y, int hmx, int hmy) {
    for ( int i=0 ; i<Math.abs(rnd.nextInt())%8+4 ; i++ ) {
      addSpark(x, y, (int)(hmx*(1.0f+(float)(rnd.nextInt()%16)/64)),
                     (int)(hmy*(0.2f+(float)(rnd.nextInt()%8)/64)));
    }
  }

  public final void getBonus() {
    addScore(bonusScore);
    if ( bonusScore < 1000 ) bonusScore += 10;
  }

  /**
   * A bonus item passes away.
   */
  public final void missBonus() {
    bonusScore /= 10;
    bonusScore >>= 1;
    bonusScore *= 10;
    if ( bonusScore < 10 ) bonusScore = 10;
  }

  public void addLeftBonus() {
    addScore(left*10000);
    left = 99;
    invCnt = extCnt = 0;
  }

  public int getScore() {
    return score;
  }

  public int getSceneScore() {
    return sceneScore;
  }
}
